import pytest

from morss.crawler import adv_get
from morss.feeds import *


def get_feed(url):
    url = 'http://localhost:8888/%s' % url
    out = adv_get(url)
    feed = parse(out['data'], url=url, encoding=out['encoding'])
    return feed

def check_feed(feed):
    # NB. time and updated not covered
    assert feed.title == '!TITLE!'
    assert feed.desc == '!DESC!'
    assert feed[0] == feed.items[0]
    assert feed[0].title == '!ITEM_TITLE!'
    assert feed[0].link == '!ITEM_LINK!'
    assert '!ITEM_DESC!' in feed[0].desc # broader test due to possible inclusion of surrounding <div> in xml
    assert '!ITEM_CONTENT!' in feed[0].content

def check_output(feed):
    output = feed.tostring()
    assert '!TITLE!' in output
    assert '!DESC!' in output
    assert '!ITEM_TITLE!' in output
    assert '!ITEM_LINK!' in output
    assert '!ITEM_DESC!' in output
    assert '!ITEM_CONTENT!' in output

def check_change(feed):
    feed.title = '!TITLE2!'
    feed.desc = '!DESC2!'
    feed[0].title = '!ITEM_TITLE2!'
    feed[0].link = '!ITEM_LINK2!'
    feed[0].desc = '!ITEM_DESC2!'
    feed[0].content = '!ITEM_CONTENT2!'

    assert feed.title == '!TITLE2!'
    assert feed.desc == '!DESC2!'
    assert feed[0].title == '!ITEM_TITLE2!'
    assert feed[0].link == '!ITEM_LINK2!'
    assert '!ITEM_DESC2!' in feed[0].desc
    assert '!ITEM_CONTENT2!' in feed[0].content

def check_add(feed):
    feed.append({
        'title': '!ITEM_TITLE3!',
        'link': '!ITEM_LINK3!',
        'desc': '!ITEM_DESC3!',
        'content': '!ITEM_CONTENT3!',
    })

    assert feed[1].title == '!ITEM_TITLE3!'
    assert feed[1].link == '!ITEM_LINK3!'
    assert '!ITEM_DESC3!' in feed[1].desc
    assert '!ITEM_CONTENT3!' in feed[1].content

each_format = pytest.mark.parametrize('url', [
    'feed-rss-channel-utf-8.txt', 'feed-atom-utf-8.txt',
    'feed-atom03-utf-8.txt', 'feed-json-utf-8.txt', 'feed-html-utf-8.txt',
    ])

each_check = pytest.mark.parametrize('check', [
    check_feed, check_output, check_change, check_add,
    ])

@each_format
@each_check
def test_parse(replay_server, url, check):
    feed = get_feed(url)
    check(feed)

@each_format
@each_check
def test_convert_rss(replay_server, url, check):
    feed = get_feed(url)
    feed = feed.convert(FeedXML)
    check(feed)

@each_format
@each_check
def test_convert_json(replay_server, url, check):
    feed = get_feed(url)
    feed = feed.convert(FeedJSON)
    check(feed)

@each_format
@each_check
def test_convert_html(replay_server, url, check):
    feed = get_feed(url)
    feed = feed.convert(FeedHTML)
    if len(feed) > 1:
        # remove the 'blank' default html item
        del feed[0]
    check(feed)

@each_format
def test_convert_csv(replay_server, url):
    # only csv output, not csv feed, check therefore differnet
    feed = get_feed(url)
    output = feed.tocsv()

    assert '!ITEM_TITLE!' in output
    assert '!ITEM_LINK!' in output
    assert '!ITEM_DESC!' in output
    assert '!ITEM_CONTENT!' in output



import pytest

from morss.crawler import *


def test_get(replay_server):
    assert get('http://localhost:8888/200-ok.txt') == b'success\r\n'

def test_adv_get(replay_server):
    assert adv_get('http://localhost:8888/200-ok.txt')['data'] == b'success\r\n'

@pytest.mark.parametrize('before,after', [
    (b'http://localhost:8888/',     'http://localhost:8888/'),
    ('localhost:8888/',             'http://localhost:8888/'),
    ('http:/localhost:8888/',       'http://localhost:8888/'),
    ('http://localhost:8888/&/',     'http://localhost:8888/&/'),
    ('http://localhost:8888/ /',    'http://localhost:8888/%20/'),
    ('http://localhost-€/€/',       'http://xn--localhost--077e/%E2%82%AC/'),
    ('http://localhost-€:8888/€/',  'http://xn--localhost--077e:8888/%E2%82%AC/'),
    ])
def test_sanitize_url(before, after):
    assert sanitize_url(before) == after

@pytest.mark.parametrize('opener', [custom_opener(), build_opener(SizeLimitHandler(500*1024))])
def test_size_limit_handler(replay_server, opener):
    assert len(opener.open('http://localhost:8888/size-1MiB.txt').read()) == 500*1024

@pytest.mark.parametrize('opener', [custom_opener(), build_opener(GZIPHandler())])
def test_gzip_handler(replay_server, opener):
    assert opener.open('http://localhost:8888/gzip.txt').read() == b'success\n'

@pytest.mark.parametrize('opener', [custom_opener(), build_opener(EncodingFixHandler())])
@pytest.mark.parametrize('url', [
    'enc-gb2312-header.txt', 'enc-gb2312-meta.txt', #'enc-gb2312-missing.txt',
    'enc-iso-8859-1-header.txt', 'enc-iso-8859-1-missing.txt',
    'enc-utf-8-header.txt',
    ])
def test_encoding_fix_handler(replay_server, opener, url):
    out = adv_get('http://localhost:8888/%s' % url)
    out = out['data'].decode(out['encoding'])
    assert 'succes' in out or 'succès' in out or '成功' in out

@pytest.mark.parametrize('opener', [custom_opener(follow='rss'), build_opener(AlternateHandler(MIMETYPE['rss']))])
def test_alternate_handler(replay_server, opener):
    assert opener.open('http://localhost:8888/alternate-abs.txt').geturl() == 'http://localhost:8888/200-ok.txt'

@pytest.mark.parametrize('opener', [custom_opener(), build_opener(HTTPEquivHandler(), HTTPRefreshHandler())])
def test_http_equiv_handler(replay_server, opener):
    assert opener.open('http://localhost:8888/meta-redirect-abs.txt').geturl() == 'http://localhost:8888/200-ok.txt'
    assert opener.open('http://localhost:8888/meta-redirect-rel.txt').geturl() == 'http://localhost:8888/200-ok.txt'
    assert opener.open('http://localhost:8888/meta-redirect-url.txt').geturl() == 'http://localhost:8888/200-ok.txt'

@pytest.mark.parametrize('opener', [custom_opener(), build_opener(HTTPAllRedirectHandler())])
def test_http_all_redirect_handler(replay_server, opener):
    assert opener.open('http://localhost:8888/308-redirect.txt').geturl() == 'http://localhost:8888/200-ok.txt'
    assert opener.open('http://localhost:8888/301-redirect-abs.txt').geturl() == 'http://localhost:8888/200-ok.txt'
    assert opener.open('http://localhost:8888/301-redirect-rel.txt').geturl() == 'http://localhost:8888/200-ok.txt'
    assert opener.open('http://localhost:8888/301-redirect-url.txt').geturl() == 'http://localhost:8888/200-ok.txt'

@pytest.mark.parametrize('opener', [custom_opener(), build_opener(HTTPRefreshHandler())])
def test_http_refresh_handler(replay_server, opener):
    assert opener.open('http://localhost:8888/header-refresh.txt').geturl() == 'http://localhost:8888/200-ok.txt'



from datetime import datetime
from glob import glob

from setuptools import setup

def get_version():
    with open('morss/__init__.py', 'r+') as file:
        lines = file.readlines()

        # look for hard coded version number
        for i in range(len(lines)):
            if lines[i].startswith('__version__'):
                version = lines[i].split('"')[1]
                break

        # create (& save) one if none found
        if version == '':
            version = datetime.now().strftime('%Y%m%d.%H%M')
            lines[i] = '__version__ = "' + version + '"\n'

            file.seek(0)
            file.writelines(lines)

        # return version number
        return version

package_name = 'morss'

setup(
    name = package_name,
    version = get_version(),
    description = 'Get full-text RSS feeds',
    long_description = open('README.md').read(),
    long_description_content_type = 'text/markdown',
    author = 'pictuga',
    author_email = 'contact@pictuga.com',
    url = 'http://morss.it/',
    project_urls = {
        'Source': 'https://git.pictuga.com/pictuga/morss',
        'Bug Tracker': 'https://github.com/pictuga/morss/issues',
    },
    license = 'AGPL v3',
    packages = [package_name],
    install_requires = ['lxml', 'bs4', 'python-dateutil', 'chardet'],
    extras_require = {
        'full': ['redis', 'diskcache', 'gunicorn', 'setproctitle'],
        'dev': ['pylint', 'pyenchant', 'pytest', 'pytest-cov'],
    },
    python_requires = '>=2.7',
    package_data = {package_name: ['feedify.ini']},
    data_files = [
        ('share/' + package_name, ['README.md', 'LICENSE']),
        ('share/' + package_name + '/www', glob('www/*.*')),
    ],
    entry_points = {
        'console_scripts': [package_name + '=' + package_name + '.__main__:main'],
    },
    scripts = ['morss-helper'],
)
